# Generated by roxygen2: do not edit by hand

export(CPEmbedding)
export(FuzzyMapperAlgo)
export(GMapperAlgo)
export(GridSearch)
export(MapperAlgo)
export(MapperCorrelation)
export(MapperPlotter)
export(cluster_cutoff_at_first_empty_bin)
export(cover_points)
export(find_best_k_for_kmeans)
export(mapperEdges)
export(mapperVertices)
export(perform_clustering)
export(recursive_gaussian_split)
export(save_mapper_png)
export(simplcial_complex)
export(to_lsfi)
export(to_lsmi)
import(foreach)
import(htmlwidgets)
import(webshot2)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme_minimal)
importFrom(graphics,hist)
importFrom(htmlwidgets,JS)
importFrom(igraph,V)
importFrom(igraph,graph.adjacency)
importFrom(inaparc,kmpp)
importFrom(mclust,Mclust)
importFrom(networkD3,forceNetwork)
importFrom(nortest,ad.test)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(ppclust,fcm)
importFrom(rlang,.data)
importFrom(stats,as.dist)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,var)
